@@ -124,14 +124,14 @@ Yield._astng_fields = ('value',)
 STMT_NODES = (
     Assign, AugAssign, Assert, Break, Class, Continue, Delete, Discard,
     ExceptHandler, Exec, For, From, Function, Global, If, Import, Pass, Print,
-    Raise, Return, TryExcept, TryFinally, While, With, Yield
+    Raise, Return, TryExcept, TryFinally, While, With
     )
 
 ALL_NODES = STMT_NODES + (
     Arguments, AssAttr, AssName, BinOp, BoolOp, Backquote,  CallFunc, Compare,
     Comprehension, Const, Decorators, DelAttr, DelName, Dict, Ellipsis,
     EmptyNode,  ExtSlice, Getattr,  GenExpr, IfExp, Index, Keyword, Lambda,
-    List,  ListComp, Module, Name, Slice, Subscript, UnaryOp, Tuple
+    List,  ListComp, Module, Name, Slice, Subscript, UnaryOp, Tuple, Yield
     )
 
 # Node  ######################################################################
@@ -180,8 +180,7 @@ class NodeNG:
         return False
 
     def statement(self):
-        """return the first parent node marked as statement node
-        """
+        """return the first parent node marked as statement node"""
         if self.is_statement:
             return self
         return self.parent.statement()
@@ -192,14 +191,13 @@ class NodeNG:
         return self.parent.frame()
 
     def scope(self):
-        """return the first node defining a new scope (i.e. Module,
-        Function, Class, Lambda but also GenExpr)
+        """return the first node defining a new scope (i.e. Module, Function,
+        Class, Lambda but also GenExpr)
         """
         return self.parent.scope()
 
     def root(self):
-        """return the root node of the tree, (i.e. a Module)
-        """
+        """return the root node of the tree, (i.e. a Module)"""
         if self.parent:
             return self.parent.root()
         return self
@@ -207,10 +205,12 @@ class NodeNG:
     def child_sequence(self, child):
         """search for the right sequence where the child lies in"""
         for field in self._astng_fields:
-            sequence = getattr(self, field)
+            node_or_sequence = getattr(self, field)
+            if node_or_sequence is child:
+                return [node_or_sequence]
             # /!\ compiler.ast Nodes have an __iter__ walking over child nodes
-            if isinstance(sequence, (tuple, list)) and child in sequence:
-                return sequence
+            if isinstance(node_or_sequence, (tuple, list)) and child in node_or_sequence:
+                return node_or_sequence
         else:
             msg = 'Could not found %s in %s\'s children'
             raise ASTNGError(msg % (repr(child), repr(self)))
@@ -218,22 +218,19 @@ class NodeNG:
     def locate_child(self, child):
         """return a 2-uple (child attribute name, sequence or node)"""
         for field in self._astng_fields:
-            sequence = getattr(self, field)
+            node_or_sequence = getattr(self, field)
             # /!\ compiler.ast Nodes have an __iter__ walking over child nodes
-            if child is sequence:
+            if child is node_or_sequence:
                 return field, child
-            if isinstance(sequence, (tuple, list)) and child in sequence:
-                return field, sequence
+            if isinstance(node_or_sequence, (tuple, list)) and child in node_or_sequence:
+                return field, node_or_sequence
         msg = 'Could not found %s in %s\'s children'
         raise ASTNGError(msg % (repr(child), repr(self)))
 
     def next_sibling(self):
-        """return the previous sibling statement
-        """
-        if isinstance(self, Module):
-            return
-        while not (self.is_statement or isinstance(self, Module)):
-            self = self.parent
+        """return the next sibling statement"""
+        if not self.is_statement:
+            return self.parent.next_sibling()
         stmts = self.parent.child_sequence(self)
         index = stmts.index(self)
         try:
@@ -242,12 +239,9 @@ class NodeNG:
             pass
 
     def previous_sibling(self):
-        """return the next sibling statement 
-        """
-        if isinstance(self, Module):
-            return
-        while not (self.is_statement or isinstance(self, Module)):
-            self = self.parent
+        """return the previous sibling statement"""
+        if not self.is_statement:
+            return self.parent.previous_sibling()
         stmts = self.parent.child_sequence(self)
         index = stmts.index(self)
         if index >= 1:
@@ -283,7 +277,7 @@ class NodeNG:
             self.tolineno = lastchild.tolineno
         assert self.fromlineno is not None, self
         assert self.tolineno is not None, self
-        
+
     def _fixed_source_line(self):
         """return the line number where the given node appears
 
@@ -302,7 +296,7 @@ class NodeNG:
                 line = _node.lineno
                 _node = _node.parent
         return line
-    
+
     def block_range(self, lineno):
         """handle block line numbers range for non block opening statements
         """
@@ -311,7 +305,7 @@ class NodeNG:
     def set_local(self, name, stmt):
         """delegate to a scoped parent handling a locals dictionary"""
         self.parent.set_local(name, stmt)
-        
+
     def nodes_of_class(self, klass, skip_klass=None):
         """return an iterator on nodes which are instance of the given class(es)
 
@@ -336,7 +330,7 @@ class NodeNG:
 
     def eq(self, value):
         return False
-    
+
     def as_string(self):
         from logilab.astng.nodes_as_string import as_string
         return as_string(self)
@@ -349,7 +343,7 @@ class NodeNG:
 
 # extend all classes instead of base Node class which is an unextendable type
 # in 2.6
-for cls in ALL_NODES: 
+for cls in ALL_NODES:
     extend_class(cls, NodeNG)
 
 INDENT = "    "
@@ -467,7 +461,7 @@ def excepthandler_set_line_info(self, lastchild):
         self.blockstart_tolineno= self.type.tolineno
     else:
         self.blockstart_tolineno= self.lineno
-    self.tolineno = lastchild.tolineno    
+    self.tolineno = lastchild.tolineno
 ExceptHandler.set_line_info = excepthandler_set_line_info
 
 def excepthandler_catch(self, exceptions):
@@ -545,6 +539,11 @@ For.block_range = elsed_block_range
 
 def try_finalbody_block_range(node, lineno):
     """handle block line numbers range for try/finally statements"""
+    child = node.body[0]
+    # py2.5 try: except: finally:
+    if (isinstance(child, TryExcept) and child.fromlineno == node.fromlineno
+        and lineno > node.fromlineno and lineno <= child.tolineno):
+        return child.block_range(lineno)
     return _elsed_block_range(node, lineno, node.finalbody)
 TryFinally.block_range = try_finalbody_block_range
 
@@ -566,4 +565,4 @@ def real_name(node, asname):
 From.real_name = real_name
 Import.real_name = real_name
 
-        
+
