@@ -97,7 +97,7 @@ class FromToLineNoTC(TestCase):
         self.assertIsInstance(return_, nodes.Return)
         self.assertEquals(return_.fromlineno, 18)
         self.assertEquals(return_.tolineno, 18)
-        
+
     def test_decorated_function_lineno(self):
         if sys.version_info < (2, 4):
             self.skip('require python >=2.4')
@@ -109,12 +109,12 @@ def function(
 ''', __name__, __file__)
         function = astng['function']
         self.assertEquals(function.fromlineno, 3) # XXX discussable, but that's what is expected by pylint right now
-        self.assertEquals(function.tolineno, 5) 
+        self.assertEquals(function.tolineno, 5)
         self.assertEquals(function.blockstart_tolineno, 4)
         self.assertEquals(function.decorators.fromlineno, 2)
-        self.assertEquals(function.decorators.tolineno, 2) 
-    
-    
+        self.assertEquals(function.decorators.tolineno, 2)
+
+
     def test_class_lineno(self):
         stmts = self.astng.body
         # on line 20:
@@ -165,30 +165,64 @@ else:
             stmt = astng.body[0]
             self.assertEquals(stmt.fromlineno, 2)
             self.assertEquals(stmt.tolineno, 6)
-            self.assertEquals(stmt.blockstart_tolineno, 2)        
+            self.assertEquals(stmt.blockstart_tolineno, 2)
             self.assertEquals(stmt.orelse[0].fromlineno, 6) # XXX
             self.assertEquals(stmt.orelse[0].tolineno, 6)
 
 
-    def test_try_lineno(self):
+    def test_try_except_lineno(self):
         astng = builder.ASTNGBuilder().string_build('''
 try:
   print a
 except:
-  break
+  pass
 else:
   print "bouh"
 ''', __name__, __file__)
         try_ = astng.body[0]
         self.assertEquals(try_.fromlineno, 2)
         self.assertEquals(try_.tolineno, 7)
-        self.assertEquals(try_.blockstart_tolineno, 2)        
+        self.assertEquals(try_.blockstart_tolineno, 2)
         self.assertEquals(try_.orelse[0].fromlineno, 7) # XXX
         self.assertEquals(try_.orelse[0].tolineno, 7)
         hdlr = try_.handlers[0]
         self.assertEquals(hdlr.fromlineno, 4)
         self.assertEquals(hdlr.tolineno, 5)
-        self.assertEquals(hdlr.blockstart_tolineno, 4)        
+        self.assertEquals(hdlr.blockstart_tolineno, 4)
+
+
+    def test_try_finally_lineno(self):
+        astng = builder.ASTNGBuilder().string_build('''
+try:
+  print a
+finally:
+  print "bouh"
+''', __name__, __file__)
+        try_ = astng.body[0]
+        self.assertEquals(try_.fromlineno, 2)
+        self.assertEquals(try_.tolineno, 5)
+        self.assertEquals(try_.blockstart_tolineno, 2)
+        self.assertEquals(try_.finalbody[0].fromlineno, 5) # XXX
+        self.assertEquals(try_.finalbody[0].tolineno, 5)
+
+
+    def test_try_finally_25_lineno(self):
+        if sys.version_info < (2, 5):
+            self.skip('require python >= 2.5')
+        astng = builder.ASTNGBuilder().string_build('''
+try:
+  print a
+except:
+  pass
+finally:
+  print "bouh"
+''', __name__, __file__)
+        try_ = astng.body[0]
+        self.assertEquals(try_.fromlineno, 2)
+        self.assertEquals(try_.tolineno, 7)
+        self.assertEquals(try_.blockstart_tolineno, 2)
+        self.assertEquals(try_.finalbody[0].fromlineno, 7) # XXX
+        self.assertEquals(try_.finalbody[0].tolineno, 7)
 
 
     def test_with_lineno(self):
@@ -203,21 +237,21 @@ with file("/tmp/pouet") as f:
         self.assertEquals(with_.fromlineno, 3)
         self.assertEquals(with_.tolineno, 4)
         self.assertEquals(with_.blockstart_tolineno, 3)
-        
 
-        
+
+
 class BuilderTC(TestCase):
-        
+
     def setUp(self):
         self.builder = builder.ASTNGBuilder()
-        
+
     def test_border_cases(self):
         """check that a file with no trailing new line is parseable"""
         self.builder.file_build('data/noendingnewline.py', 'data.noendingnewline')
         self.assertRaises(builder.ASTNGBuildingException,
                           self.builder.file_build, 'data/inexistant.py', 'whatever')
-        
-    def test_inspect_build(self):
+
+    def test_inspect_build0(self):
         """test astng tree build from a living object"""
         builtin_astng = MANAGER.astng_from_module_name('__builtin__')
         fclass = builtin_astng['file']
@@ -226,7 +260,7 @@ class BuilderTC(TestCase):
         self.assert_('read' in fclass)
         self.assert_(fclass.newstyle)
         self.assert_(fclass.pytype(), '__builtin__.type')
-        self.assert_(isinstance(fclass['read'], nodes.Function))
+        self.assertIsInstance(fclass['read'], nodes.Function)
         # check builtin function has args.args == None
         dclass = builtin_astng['dict']
         self.assertEquals(dclass['has_key'].args.args, None)
@@ -240,18 +274,17 @@ class BuilderTC(TestCase):
         # check property has __init__
         pclass = builtin_astng['property']
         self.assert_('__init__' in pclass)
-        self.failUnless(isinstance(builtin_astng['None'], nodes.Const), builtin_astng['None'])
-        self.failUnless(isinstance(builtin_astng['True'], nodes.Const), builtin_astng['True'])
-        self.failUnless(isinstance(builtin_astng['False'], nodes.Const), builtin_astng['False'])
-        self.failUnless(isinstance(builtin_astng['Exception'], nodes.From), builtin_astng['Exception'])
-        self.failUnless(isinstance(builtin_astng['NotImplementedError'], nodes.From))
-       
-        # 
+        self.assertIsInstance(builtin_astng['None'], nodes.Const)
+        self.assertIsInstance(builtin_astng['True'], nodes.Const)
+        self.assertIsInstance(builtin_astng['False'], nodes.Const)
+        self.assertIsInstance(builtin_astng['Exception'], nodes.From)
+        self.assertIsInstance(builtin_astng['NotImplementedError'], nodes.From)
+
+    def test_inspect_build1(self):
         time_astng = MANAGER.astng_from_module_name('time')
         self.assert_(time_astng)
-        unittest_astng = self.builder.inspect_build(unittest)
+        self.assertEquals(time_astng['time'].args.defaults, [])
 
-        
     def test_inspect_build2(self):
         """test astng tree build from a living object"""
         try:
@@ -264,6 +297,9 @@ class BuilderTC(TestCase):
             # this one is failing since DateTimeType.__module__ = 'builtins' !
             #dt_astng.getattr('DateTimeType')
 
+    def test_inspect_build3(self):
+        unittest_astng = self.builder.inspect_build(unittest)
+
     def test_inspect_build_instance(self):
         """test astng tree build from a living object"""
         import exceptions
@@ -281,19 +317,19 @@ class BuilderTC(TestCase):
 
     def test_inspect_build_type_object(self):
         builtin_astng = MANAGER.astng_from_module_name('__builtin__')
-        
+
         infered = list(builtin_astng.igetattr('object'))
         self.assertEquals(len(infered), 1)
         infered = infered[0]
         self.assertEquals(infered.name, 'object')
         as_string(infered)
-            
+
         infered = list(builtin_astng.igetattr('type'))
         self.assertEquals(len(infered), 1)
         infered = infered[0]
         self.assertEquals(infered.name, 'type')
         as_string(infered)
-        
+
     def test_package_name(self):
         """test base properties and method of a astng module"""
         datap = self.builder.file_build('data/__init__.py', 'data')
@@ -304,7 +340,7 @@ class BuilderTC(TestCase):
         self.assertEquals(datap.package, 1)
 
     def test_yield_parent(self):
-        """check if we removed occasional discard nodes as yield parent"""
+        """check if we added discard nodes as yield parent (w/ compiler)"""
         data = """
 def yiell():
     yield 0
@@ -313,9 +349,12 @@ def yiell():
 """
         func = self.builder.string_build(data).body[0]
         self.assertIsInstance(func, nodes.Function)
-        yill = func.body[0]
-        self.assertIsInstance(yill, nodes.Yield)
-        self.assertIsInstance(func.body[1].body[0], nodes.Yield)
+        stmt = func.body[0]
+        self.assertIsInstance(stmt, nodes.Discard)
+        self.assertIsInstance(stmt.value, nodes.Yield)
+        self.assertIsInstance(func.body[1].body[0], nodes.Discard)
+        self.assertIsInstance(func.body[1].body[0].value, nodes.Yield)
+
     def test_object(self):
         obj_astng = self.builder.inspect_build(object)
         self.failUnless('__setattr__' in obj_astng)
@@ -324,13 +363,13 @@ def yiell():
         data = '''
 class A:
     "old style"
-        
+
 class B(A):
     "old style"
-        
+
 class C(object):
     "new style"
-        
+
 class D(C):
     "new style"
 
@@ -338,7 +377,7 @@ __metaclass__ = type
 
 class E(A):
     "old style"
-    
+
 class F:
     "new style"
 '''
@@ -361,10 +400,10 @@ def update_global():
 def global_no_effect():
     global CSTE2
     print CSTE
-'''        
+'''
         astng = self.builder.string_build(data, __name__, __file__)
         self.failUnlessEqual(len(astng.getattr('CSTE')), 2)
-        self.failUnless(isinstance(astng.getattr('CSTE')[0], nodes.AssName))
+        self.assertIsInstance(astng.getattr('CSTE')[0], nodes.AssName)
         self.failUnlessEqual(astng.getattr('CSTE')[0].fromlineno, 2)
         self.failUnlessEqual(astng.getattr('CSTE')[1].fromlineno, 6)
         self.assertRaises(nodes.NotFoundError,
@@ -396,7 +435,7 @@ def global_no_effect():
             self.failIf(n.scope() is astng)
             self.failUnlessEqual([i.__class__ for i in n.infer()],
                                  [YES.__class__])
-        
+
 class FileBuildTC(TestCase):
 
     def setUp(self):
@@ -418,7 +457,7 @@ class FileBuildTC(TestCase):
         self.assert_(not module.is_statement)
         self.assertEquals(module.statement(), module)
         self.assertEquals(module.statement(), module)
-        
+
     def test_module_locals(self):
         """test the 'locals' dictionary of a astng module"""
         module = self.module
@@ -428,7 +467,7 @@ class FileBuildTC(TestCase):
         keys.sort()
         should = ['MY_DICT', 'YO', 'YOUPI',
                 '__revision__',  'global_access','modutils', 'nested_args',
-                 'os', 'redirect', 'spawn', 'REDIRECT', 'LocalsVisitor', 
+                 'os', 'redirect', 'spawn', 'REDIRECT', 'LocalsVisitor',
                 'ASTWalker', 'ASTVisitor',  'extend_class']
         should.sort()
         self.assertEquals(keys, should)
@@ -454,7 +493,7 @@ class FileBuildTC(TestCase):
         keys = _locals.keys()
         keys.sort()
         self.assertEquals(keys, ['i', 'key', 'local', 'val'])
-        
+
     def test_class_base_props(self):
         """test base properties and method of a astng class"""
         module = self.module
@@ -468,7 +507,7 @@ class FileBuildTC(TestCase):
         self.assertEquals(klass.root(), module)
         self.assertEquals(klass.basenames, [])
         self.assertEquals(klass.newstyle, False)
-                        
+
     def test_class_locals(self):
         """test the 'locals' dictionary of a astng class"""
         module = self.module
@@ -496,7 +535,7 @@ class FileBuildTC(TestCase):
         klass2 = module['YOUPI']
         self.assertEquals(klass1.basenames, [])
         self.assertEquals(klass2.basenames, ['YO'])
-        
+
     def test_method_base_props(self):
         """test base properties and method of a astng method"""
         klass2 = self.module['YOUPI']
@@ -526,7 +565,7 @@ class FileBuildTC(TestCase):
         keys.sort()
         self.assertEquals(keys, ['a', 'autre', 'b', 'local', 'self'])
 
-        
+
 class ModuleBuildTC(FileBuildTC):
 
     def setUp(self):
@@ -535,10 +574,10 @@ class ModuleBuildTC(FileBuildTC):
 
 
 class MoreTC(TestCase):
-        
+
     def setUp(self):
         self.builder = builder.ASTNGBuilder()
-        
+
     def test_infered_build(self):
         code = '''class A: pass
 A.type = "class"
@@ -553,7 +592,7 @@ A.ass_type = A_ass_type
         lclass = lclass[0]
         self.assert_('ass_type' in lclass.locals, lclass.locals.keys())
         self.assert_('type' in lclass.locals.keys())
-        
+
 #     def test_1(self):
 #         from logilab import astng
 #         import compiler
@@ -602,6 +641,6 @@ def visit_if(self, node):
 '''
         astng = self.builder.string_build(code)
         self.failUnless('body' in astng['visit_if'].locals)
-        
+
 if __name__ == '__main__':
     unittest_main()
